@@ -6,11 +6,11 @@ transparent proxied access to the endpoint of an association object.
 See the example ``examples/association/proxied_association.py``.
 """
 
-from sqlalchemy.orm.attributes import InstrumentedList
+import weakref, itertools
 import sqlalchemy.exceptions as exceptions
 import sqlalchemy.orm as orm
 import sqlalchemy.util as util
-import weakref
+
 
 def association_proxy(targetcollection, attr, **kw):
     """Convenience function for use in mapped classes.  Implements a Python
@@ -61,7 +61,7 @@ class AssociationProxy(object):
     on an object."""
 
     def __init__(self, targetcollection, attr, creator=None,
-                 proxy_factory=None, proxy_bulk_set=None):
+                 getset_factory=None, proxy_factory=None, proxy_bulk_set=None):
         """Arguments are:
 
           targetcollection
@@ -84,6 +84,17 @@ class AssociationProxy(object):
             If you want to construct instances differently, supply a 'creator'
             function that takes arguments as above and returns instances.
 
+          getset_factory
+            Optional.  Proxied attribute access is automatically handled
+            by routines that get and set values based on the `attr` argument
+            for this proxy.
+
+            If you would like to customize this behavior, you may supply a
+            `getset_factory` callable that produces a tuple of `getter` and
+            `setter` functions.  The factory is called with two arguments,
+            the abstract type of the underlying collection and this proxy
+            instance.
+
           proxy_factory
             Optional.  The type of collection to emulate is determined by
             sniffing the target collection.  If your collection type can't be
@@ -95,10 +106,11 @@ class AssociationProxy(object):
             Optional, use with proxy_factory.  See the _set() method for
             details.
         """
-        
+
         self.target_collection = targetcollection # backwards compat name...
         self.value_attr = attr
         self.creator = creator
+        self.getset_factory = getset_factory
         self.proxy_factory = proxy_factory
         self.proxy_bulk_set = proxy_bulk_set
 
@@ -109,7 +121,7 @@ class AssociationProxy(object):
         self.collection_class = None
 
     def _get_property(self):
-        return orm.class_mapper(self.owning_class).props[self.target_collection]
+        return orm.class_mapper(self.owning_class).get_property(self.target_collection)
 
     def _target_class(self):
         return self._get_property().mapper.class_
@@ -129,16 +141,18 @@ class AssociationProxy(object):
                     "scope")
             return getattr(obj, target)
         return lazy_collection
-        
+
     def __get__(self, obj, class_):
         if obj is None:
             self.owning_class = class_
             return
         elif self.scalar is None:
             self.scalar = self._target_is_scalar()
+            if self.scalar:
+                self._initialize_scalar_accessors()
 
         if self.scalar:
-            return getattr(getattr(obj, self.target_collection), self.value_attr)
+            return self._scalar_get(getattr(obj, self.target_collection))
         else:
             try:
                 return getattr(obj, self.key)
@@ -150,6 +164,8 @@ class AssociationProxy(object):
     def __set__(self, obj, values):
         if self.scalar is None:
             self.scalar = self._target_is_scalar()
+            if self.scalar:
+                self._initialize_scalar_accessors()
 
         if self.scalar:
             creator = self.creator and self.creator or self.target_class
@@ -157,39 +173,48 @@ class AssociationProxy(object):
             if target is None:
                 setattr(obj, self.target_collection, creator(values))
             else:
-                setattr(target, self.value_attr, values)
+                self._scalar_set(target, values)
         else:
             proxy = self.__get__(obj, None)
-            proxy.clear()
-            self._set(proxy, values)
+            if proxy is not values:
+                proxy.clear()
+                self._set(proxy, values)
 
     def __delete__(self, obj):
         delattr(obj, self.key)
 
+    def _initialize_scalar_accessors(self):
+        if self.getset_factory:
+            get, set = self.getset_factory(None, self)
+        else:
+            get, set = self._default_getset(None)
+        self._scalar_get, self._scalar_set = get, set
+
+    def _default_getset(self, collection_class):
+        attr = self.value_attr
+        getter = util.attrgetter(attr)
+        if collection_class is dict:
+            setter = lambda o, k, v: setattr(o, attr, v)
+        else:
+            setter = lambda o, v: setattr(o, attr, v)
+        return getter, setter
+
     def _new(self, lazy_collection):
         creator = self.creator and self.creator or self.target_class
-
-        # Prefer class typing here to spot dicts with the required append()
-        # method.
-        collection = lazy_collection()
-        if isinstance(collection.data, dict):
-            self.collection_class = dict
-        else:
-            self.collection_class = util.duck_type_collection(collection.data)
-        del collection
+        self.collection_class = util.duck_type_collection(lazy_collection())
 
         if self.proxy_factory:
             return self.proxy_factory(lazy_collection, creator, self.value_attr)
 
-        value_attr = self.value_attr
-        getter = lambda o: getattr(o, value_attr)
-        setter = lambda o, v: setattr(o, value_attr, v)
-        
+        if self.getset_factory:
+            getter, setter = self.getset_factory(self.collection_class, self)
+        else:
+            getter, setter = self._default_getset(self.collection_class)
+
         if self.collection_class is list:
             return _AssociationList(lazy_collection, creator, getter, setter)
         elif self.collection_class is dict:
-            kv_setter = lambda o, k, v: setattr(o, value_attr, v)
-            return _AssociationDict(lazy_collection, creator, getter, kv_setter)
+            return _AssociationDict(lazy_collection, creator, getter, setter)
         elif self.collection_class is util.Set:
             return _AssociationSet(lazy_collection, creator, getter, setter)
         else:
@@ -223,7 +248,7 @@ class _AssociationList(object):
         lazy_collection
           A callable returning a list-based collection of entities (usually
           an object attribute managed by a SQLAlchemy relation())
-          
+
         creator
           A function that creates new target entities.  Given one parameter:
           value.  The assertion is assumed:
@@ -267,15 +292,42 @@ class _AssociationList(object):
 
     def __getitem__(self, index):
         return self._get(self.col[index])
-    
+
     def __setitem__(self, index, value):
-        self._set(self.col[index], value)
+        if not isinstance(index, slice):
+            self._set(self.col[index], value)
+        else:
+            if index.stop is None:
+                stop = len(self)
+            elif index.stop < 0:
+                stop = len(self) + index.stop
+            else:
+                stop = index.stop
+            step = index.step or 1
+
+            rng = range(index.start or 0, stop, step)
+            if step == 1:
+                for i in rng:
+                    del self[index.start]
+                i = index.start
+                for item in value:
+                    self.insert(i, item)
+                    i += 1
+            else:
+                if len(value) != len(rng):
+                    raise ValueError(
+                        "attempt to assign sequence of size %s to "
+                        "extended slice of size %s" % (len(value),
+                                                       len(rng)))
+                for i, item in zip(rng, value):
+                    self._set(self.col[i], item)
 
     def __delitem__(self, index):
         del self.col[index]
 
     def __contains__(self, value):
         for member in self.col:
+            # testlib.pragma exempt:__eq__
             if self._get(member) == value:
                 return True
         return False
@@ -291,9 +343,13 @@ class _AssociationList(object):
         del self.col[start:end]
 
     def __iter__(self):
-        """Iterate over proxied values.  For the actual domain objects,
-        iterate over .col instead or just use the underlying collection
-        directly from its property on the parent."""
+        """Iterate over proxied values.
+
+        For the actual domain objects, iterate over .col instead or
+        just use the underlying collection directly from its property
+        on the parent.
+        """
+
         for member in self.col:
             yield self._get(member)
         raise StopIteration
@@ -304,6 +360,10 @@ class _AssociationList(object):
         item = self._create(value, **kw)
         self.col.append(item)
 
+    def count(self, value):
+        return sum([1 for _ in
+                    itertools.ifilter(lambda v: v == value, iter(self))])
+
     def extend(self, values):
         for v in values:
             self.append(v)
@@ -311,6 +371,26 @@ class _AssociationList(object):
     def insert(self, index, value):
         self.col[index:index] = [self._create(value)]
 
+    def pop(self, index=-1):
+        return self.getter(self.col.pop(index))
+
+    def remove(self, value):
+        for i, val in enumerate(self):
+            if val == value:
+                del self.col[i]
+                return
+        raise ValueError("value not in list")
+
+    def reverse(self):
+        """Not supported, use reversed(mylist)"""
+
+        raise NotImplementedError
+
+    def sort(self):
+        """Not supported, use sorted(mylist)"""
+
+        raise NotImplementedError
+
     def clear(self):
         del self.col[0:len(self.col)]
 
@@ -322,13 +402,44 @@ class _AssociationList(object):
     def __ge__(self, other): return list(self) >= other
     def __cmp__(self, other): return cmp(list(self), other)
 
+    def __add__(self, iterable):
+        try:
+            other = list(iterable)
+        except TypeError:
+            return NotImplemented
+        return list(self) + other
+    __radd__ = __add__
+
+    def __mul__(self, n):
+        if not isinstance(n, int):
+            return NotImplemented
+        return list(self) * n
+    __rmul__ = __mul__
+
+    def __iadd__(self, iterable):
+        self.extend(iterable)
+        return self
+
+    def __imul__(self, n):
+        # unlike a regular list *=, proxied __imul__ will generate unique
+        # backing objects for each copy.  *= on proxied lists is a bit of
+        # a stretch anyhow, and this interpretation of the __imul__ contract
+        # is more plausibly useful than copying the backing objects.
+        if not isinstance(n, int):
+            return NotImplemented
+        if n == 0:
+            self.clear()
+        elif n > 1:
+            self.extend(list(self) * (n - 1))
+        return self
+
     def copy(self):
         return list(self)
 
     def __repr__(self):
         return repr(list(self))
 
-    def hash(self):
+    def __hash__(self):
         raise TypeError("%s objects are unhashable" % type(self).__name__)
 
 _NotProvided = object()
@@ -342,7 +453,7 @@ class _AssociationDict(object):
         lazy_collection
           A callable returning a dict-based collection of entities (usually
           an object attribute managed by a SQLAlchemy relation())
-          
+
         creator
           A function that creates new target entities.  Given two parameters:
           key and value.  The assertion is assumed:
@@ -384,7 +495,7 @@ class _AssociationDict(object):
 
     def __getitem__(self, key):
         return self._get(self.col[key])
-    
+
     def __setitem__(self, key, value):
         if key in self.col:
             self._set(self.col[key], key, value)
@@ -395,6 +506,7 @@ class _AssociationDict(object):
         del self.col[key]
 
     def __contains__(self, key):
+        # testlib.pragma exempt:__hash__
         return key in self.col
     has_key = __contains__
 
@@ -457,7 +569,7 @@ class _AssociationDict(object):
     def popitem(self):
         item = self.col.popitem()
         return (item[0], self._get(item[1]))
-    
+
     def update(self, *a, **kw):
         if len(a) > 1:
             raise TypeError('update expected at most 1 arguments, got %i' %
@@ -476,7 +588,7 @@ class _AssociationDict(object):
     def copy(self):
         return dict(self.items())
 
-    def hash(self):
+    def __hash__(self):
         raise TypeError("%s objects are unhashable" % type(self).__name__)
 
 class _AssociationSet(object):
@@ -489,7 +601,7 @@ class _AssociationSet(object):
         collection
           A callable returning a set-based collection of entities (usually an
           object attribute managed by a SQLAlchemy relation())
-          
+
         creator
           A function that creates new target entities.  Given one parameter:
           value.  The assertion is assumed:
@@ -531,6 +643,7 @@ class _AssociationSet(object):
 
     def __contains__(self, value):
         for member in self.col:
+            # testlib.pragma exempt:__eq__
             if self._get(member) == value:
                 return True
         return False
@@ -545,9 +658,7 @@ class _AssociationSet(object):
 
     def add(self, value):
         if value not in self:
-            # must shove this through InstrumentedList.append() which will
-            # eventually call the collection_class .add()
-            self.col.append(self._create(value))
+            self.col.add(self._create(value))
 
     # for discard and remove, choosing a more expensive check strategy rather
     # than call self.creator()
@@ -567,19 +678,19 @@ class _AssociationSet(object):
     def pop(self):
         if not self.col:
             raise KeyError('pop from an empty set')
-        # grumble, pop() is borked on InstrumentedList (#548)
-        if isinstance(self.col, InstrumentedList):
-            member = list(self.col)[0]
-            self.col.remove(member)
-        else:
-            member = self.col.pop()
+        member = self.col.pop()
         return self._get(member)
 
     def update(self, other):
         for value in other:
             self.add(value)
 
-    __ior__ = update
+    def __ior__(self, other):
+        if util.duck_type_collection(other) is not set:
+            return NotImplemented
+        for value in other:
+            self.add(value)
+        return self
 
     def _set(self):
         return util.Set(iter(self))
@@ -598,7 +709,12 @@ class _AssociationSet(object):
         for value in other:
             self.discard(value)
 
-    __isub__ = difference_update
+    def __isub__(self, other):
+        if util.duck_type_collection(other) is not set:
+            return NotImplemented
+        for value in other:
+            self.discard(value)
+        return self
 
     def intersection(self, other):
         return util.Set(self).intersection(other)
@@ -615,7 +731,18 @@ class _AssociationSet(object):
         for value in add:
             self.add(value)
 
-    __iand__ = intersection_update
+    def __iand__(self, other):
+        if util.duck_type_collection(other) is not set:
+            return NotImplemented
+        want, have = self.intersection(other), util.Set(self)
+
+        remove, add = have - want, want - have
+
+        for value in remove:
+            self.remove(value)
+        for value in add:
+            self.add(value)
+        return self
 
     def symmetric_difference(self, other):
         return util.Set(self).symmetric_difference(other)
@@ -632,14 +759,25 @@ class _AssociationSet(object):
         for value in add:
             self.add(value)
 
-    __ixor__ = symmetric_difference_update
+    def __ixor__(self, other):
+        if util.duck_type_collection(other) is not set:
+            return NotImplemented
+        want, have = self.symmetric_difference(other), util.Set(self)
+
+        remove, add = have - want, want - have
+
+        for value in remove:
+            self.remove(value)
+        for value in add:
+            self.add(value)
+        return self
 
     def issubset(self, other):
         return util.Set(self).issubset(other)
-    
+
     def issuperset(self, other):
         return util.Set(self).issuperset(other)
-                
+
     def clear(self):
         self.col.clear()
 
@@ -656,5 +794,5 @@ class _AssociationSet(object):
     def __repr__(self):
         return repr(util.Set(self))
 
-    def hash(self):
+    def __hash__(self):
         raise TypeError("%s objects are unhashable" % type(self).__name__)
