@@ -1,29 +1,24 @@
-from testbase import PersistTest
+import testbase
 import sqlalchemy.topological as topological
-import unittest, sys, os
 from sqlalchemy import util
-
-# TODO:  need assertion conditions in this suite
+from testlib import *
 
 
-class DependencySorter(topological.QueueDependencySorter):pass
-    
-        
 class DependencySortTest(PersistTest):
     def assert_sort(self, tuples, node, collection=None):
         print str(node)
         def assert_tuple(tuple, node):
-            if node.cycles:
-                cycles = [i.item for i in node.cycles]
+            if node[1]:
+                cycles = node[1]
             else:
                 cycles = []
-            if tuple[0] is node.item or tuple[0] in cycles:
+            if tuple[0] is node[0] or tuple[0] in cycles:
                 tuple.pop()
-                if tuple[0] is node.item or tuple[0] in cycles:
+                if tuple[0] is node[0] or tuple[0] in cycles:
                     return
-            elif len(tuple) > 1 and tuple[1] is node.item:
-                assert False, "Tuple not in dependency tree: " + str(tuple)
-            for c in node.children:
+            elif len(tuple) > 1 and tuple[1] is node[0]:
+                assert False, "Tuple not in dependency tree: " + str(tuple) + " " + str(node)
+            for c in node[2]:
                 assert_tuple(tuple, c)
         
         for tuple in tuples:
@@ -33,12 +28,12 @@ class DependencySortTest(PersistTest):
             collection = []
         items = util.Set()
         def assert_unique(node):
-            for item in [n.item for n in node.cycles or [node,]]:
+            for item in [i for i in node[1] or [node[0]]]:
                 assert item not in items
                 items.add(item)
                 if item in collection:
                     collection.remove(item)
-            for c in node.children:
+            for c in node[2]:
                 assert_unique(c)
         assert_unique(node)
         assert len(collection) == 0
@@ -63,7 +58,7 @@ class DependencySortTest(PersistTest):
             (node4, subnode3),
             (node4, subnode4)
         ]
-        head = DependencySorter(tuples, []).sort()
+        head = topological.sort_as_tree(tuples, [])
         self.assert_sort(tuples, head)
 
     def testsort2(self):
@@ -81,7 +76,7 @@ class DependencySortTest(PersistTest):
             (node5, node6),
             (node6, node2)
         ]
-        head = DependencySorter(tuples, [node7]).sort()
+        head = topological.sort_as_tree(tuples, [node7])
         self.assert_sort(tuples, head, [node7])
 
     def testsort3(self):
@@ -94,9 +89,9 @@ class DependencySortTest(PersistTest):
             (node3, node2),
             (node1,node3)
         ]
-        head1 = DependencySorter(tuples, [node1, node2, node3]).sort()
-        head2 = DependencySorter(tuples, [node3, node1, node2]).sort()
-        head3 = DependencySorter(tuples, [node3, node2, node1]).sort()
+        head1 = topological.sort_as_tree(tuples, [node1, node2, node3])
+        head2 = topological.sort_as_tree(tuples, [node3, node1, node2])
+        head3 = topological.sort_as_tree(tuples, [node3, node2, node1])
         
         # TODO: figure out a "node == node2" function
         #self.assert_(str(head1) == str(head2) == str(head3))
@@ -115,7 +110,7 @@ class DependencySortTest(PersistTest):
             (node1, node3),
             (node3, node2)
         ]
-        head = DependencySorter(tuples, []).sort()
+        head = topological.sort_as_tree(tuples, [])
         self.assert_sort(tuples, head)
 
     def testsort5(self):
@@ -138,7 +133,7 @@ class DependencySortTest(PersistTest):
             node3,
             node4
         ]
-        head = DependencySorter(tuples, allitems).sort()
+        head = topological.sort_as_tree(tuples, allitems, with_cycles=True)
         self.assert_sort(tuples, head)
 
     def testcircular(self):
@@ -155,7 +150,8 @@ class DependencySortTest(PersistTest):
             (node3, node1),
             (node4, node1)
         ]
-        head = DependencySorter(tuples, []).sort(allow_all_cycles=True)
+        allitems = [node1, node2, node3, node4]
+        head = topological.sort_as_tree(tuples, allitems, with_cycles=True)
         self.assert_sort(tuples, head)
         
     def testcircular2(self):
@@ -172,17 +168,22 @@ class DependencySortTest(PersistTest):
             (node3, node2),
             (node2, node3)
         ]
-        head = DependencySorter(tuples, []).sort(allow_all_cycles=True)
+        head = topological.sort_as_tree(tuples, [], with_cycles=True)
         self.assert_sort(tuples, head)
     
     def testcircular3(self):
         nodes = {}
         tuples = [('Question', 'Issue'), ('ProviderService', 'Issue'), ('Provider', 'Question'), ('Question', 'Provider'), ('ProviderService', 'Question'), ('Provider', 'ProviderService'), ('Question', 'Answer'), ('Issue', 'Question')]
-        head = DependencySorter(tuples, []).sort(allow_all_cycles=True)
+        head = topological.sort_as_tree(tuples, [], with_cycles=True)
         self.assert_sort(tuples, head)
         
+    def testbigsort(self):
+        tuples = []
+        for i in range(0,1500, 2):
+            tuples.append((i, i+1))
+        head = topological.sort_as_tree(tuples, [])
             
             
             
 if __name__ == "__main__":
-    unittest.main()
+    testbase.main()
