@@ -1,10 +1,13 @@
-import testbase, tables
-import unittest, sys, datetime
+import testbase
 
-from sqlalchemy.ext.sessioncontext import SessionContext
 from sqlalchemy import *
+from sqlalchemy import exceptions
+from sqlalchemy.orm import *
+from sqlalchemy.ext.sessioncontext import SessionContext
+from testlib import *
+import testlib.tables as tables
 
-class O2MCascadeTest(testbase.AssertMixin):
+class O2MCascadeTest(AssertMixin):
     def tearDown(self):
         tables.delete()
 
@@ -20,34 +23,34 @@ class O2MCascadeTest(testbase.AssertMixin):
             orders = relation(
                 mapper(tables.Order, tables.orders, properties = dict (
                     items = relation(mapper(tables.Item, tables.orderitems), lazy=True, uselist =True, cascade="all, delete-orphan")
-                )), 
+                )),
                 lazy = True, uselist = True, cascade="all, delete-orphan")
         ))
 
     def setUp(self):
         global data
         data = [tables.User,
-            {'user_name' : 'ed', 
+            {'user_name' : 'ed',
                 'address' : (tables.Address, {'email_address' : 'foo@bar.com'}),
                 'orders' : (tables.Order, [
-                    {'description' : 'eds 1st order', 'items' : (tables.Item, [{'item_name' : 'eds o1 item'}, {'item_name' : 'eds other o1 item'}])}, 
+                    {'description' : 'eds 1st order', 'items' : (tables.Item, [{'item_name' : 'eds o1 item'}, {'item_name' : 'eds other o1 item'}])},
                     {'description' : 'eds 2nd order', 'items' : (tables.Item, [{'item_name' : 'eds o2 item'}, {'item_name' : 'eds other o2 item'}])}
                  ])
             },
-            {'user_name' : 'jack', 
+            {'user_name' : 'jack',
                 'address' : (tables.Address, {'email_address' : 'jack@jack.com'}),
                 'orders' : (tables.Order, [
                     {'description' : 'jacks 1st order', 'items' : (tables.Item, [{'item_name' : 'im a lumberjack'}, {'item_name' : 'and im ok'}])}
                  ])
             },
-            {'user_name' : 'foo', 
+            {'user_name' : 'foo',
                 'address' : (tables.Address, {'email_address': 'hi@lala.com'}),
                 'orders' : (tables.Order, [
-                    {'description' : 'foo order', 'items' : (tables.Item, [])}, 
-                    {'description' : 'foo order 2', 'items' : (tables.Item, [{'item_name' : 'hi'}])}, 
+                    {'description' : 'foo order', 'items' : (tables.Item, [])},
+                    {'description' : 'foo order 2', 'items' : (tables.Item, [{'item_name' : 'hi'}])},
                     {'description' : 'foo order three', 'items' : (tables.Item, [{'item_name' : 'there'}])}
                 ])
-            }        
+            }
         ]
 
         sess = create_session()
@@ -87,7 +90,7 @@ class O2MCascadeTest(testbase.AssertMixin):
         assert o2 in sess
         sess.flush()
         sess.clear()
-        
+
         u = sess.query(tables.User).get(u.user_id)
         o3 = tables.Order()
         o3.description='order3'
@@ -97,7 +100,7 @@ class O2MCascadeTest(testbase.AssertMixin):
         assert o3 in sess
         assert o4 in sess
         sess.flush()
-        
+
         o5 = tables.Order()
         o5.description='order5'
         sess.save(o5)
@@ -106,13 +109,13 @@ class O2MCascadeTest(testbase.AssertMixin):
             assert False
         except exceptions.FlushError, e:
             assert "is an orphan" in str(e)
-        
-        
+
+
     def testdelete(self):
         sess = create_session()
-        l = sess.query(tables.User).select()
+        l = sess.query(tables.User).all()
         for u in l:
-            self.echo( repr(u.orders))
+            print repr(u.orders)
         self.assert_result(l, data[0], *data[1:])
 
         ids = (l[0].user_id, l[2].user_id)
@@ -120,14 +123,14 @@ class O2MCascadeTest(testbase.AssertMixin):
         sess.delete(l[2])
 
         sess.flush()
-        assert tables.orders.count(tables.orders.c.user_id.in_(*ids)).scalar() == 0
-        assert tables.orderitems.count(tables.orders.c.user_id.in_(*ids)  &(tables.orderitems.c.order_id==tables.orders.c.order_id)).scalar() == 0
-        assert tables.addresses.count(tables.addresses.c.user_id.in_(*ids)).scalar() == 0
-        assert tables.users.count(tables.users.c.user_id.in_(*ids)).scalar() == 0
+        assert tables.orders.count(tables.orders.c.user_id.in_(ids)).scalar() == 0
+        assert tables.orderitems.count(tables.orders.c.user_id.in_(ids)  &(tables.orderitems.c.order_id==tables.orders.c.order_id)).scalar() == 0
+        assert tables.addresses.count(tables.addresses.c.user_id.in_(ids)).scalar() == 0
+        assert tables.users.count(tables.users.c.user_id.in_(ids)).scalar() == 0
 
     def testdelete2(self):
         """test that unloaded collections are still included in a delete-cascade by default."""
-        
+
         sess = create_session()
         u = sess.query(tables.User).get_by(user_name='ed')
         # assert 'addresses' collection not loaded
@@ -141,7 +144,7 @@ class O2MCascadeTest(testbase.AssertMixin):
         """test that cascade only reaches instances that are still part of the collection,
         not those that have been removed"""
         sess = create_session()
-        
+
         u = tables.User()
         u.user_name = 'newuser'
         o = tables.Order()
@@ -149,51 +152,51 @@ class O2MCascadeTest(testbase.AssertMixin):
         u.orders.append(o)
         sess.save(u)
         sess.flush()
-        
+
         u.orders.remove(o)
         sess.delete(u)
         assert u in sess.deleted
         assert o not in sess.deleted
-    
+
 
     def testorphan(self):
         sess = create_session()
-        l = sess.query(tables.User).select()
+        l = sess.query(tables.User).all()
         jack = l[1]
         jack.orders[:] = []
 
         ids = [jack.user_id]
-        self.assert_(tables.orders.count(tables.orders.c.user_id.in_(*ids)).scalar() == 1)
-        self.assert_(tables.orderitems.count(tables.orders.c.user_id.in_(*ids)  &(tables.orderitems.c.order_id==tables.orders.c.order_id)).scalar() == 2)
+        self.assert_(tables.orders.count(tables.orders.c.user_id.in_(ids)).scalar() == 1)
+        self.assert_(tables.orderitems.count(tables.orders.c.user_id.in_(ids)  &(tables.orderitems.c.order_id==tables.orders.c.order_id)).scalar() == 2)
 
         sess.flush()
 
-        self.assert_(tables.orders.count(tables.orders.c.user_id.in_(*ids)).scalar() == 0)
-        self.assert_(tables.orderitems.count(tables.orders.c.user_id.in_(*ids)  &(tables.orderitems.c.order_id==tables.orders.c.order_id)).scalar() == 0)
+        self.assert_(tables.orders.count(tables.orders.c.user_id.in_(ids)).scalar() == 0)
+        self.assert_(tables.orderitems.count(tables.orders.c.user_id.in_(ids)  &(tables.orderitems.c.order_id==tables.orders.c.order_id)).scalar() == 0)
 
 
-class M2OCascadeTest(testbase.AssertMixin):
+class M2OCascadeTest(AssertMixin):
     def tearDown(self):
         ctx.current.clear()
         for t in metadata.table_iterator(reverse=True):
             t.delete().execute()
-            
+
     def tearDownAll(self):
         clear_mappers()
         metadata.drop_all()
-        
+
     def setUpAll(self):
         global ctx, data, metadata, User, Pref, Extra
         ctx = SessionContext(create_session)
         metadata = MetaData(testbase.db)
-        extra = Table("extra", metadata, 
+        extra = Table("extra", metadata,
             Column("extra_id", Integer, Sequence("extra_id_seq", optional=True), primary_key=True),
             Column("prefs_id", Integer, ForeignKey("prefs.prefs_id"))
         )
-        prefs = Table('prefs', metadata, 
+        prefs = Table('prefs', metadata,
             Column('prefs_id', Integer, Sequence('prefs_id_seq', optional=True), primary_key=True),
             Column('prefs_data', String(40)))
-            
+
         users = Table('users', metadata,
             Column('user_id', Integer, Sequence('user_id_seq', optional=True), primary_key = True),
             Column('user_name', String(40)),
@@ -233,6 +236,7 @@ class M2OCascadeTest(testbase.AssertMixin):
         ctx.current.flush()
         ctx.current.clear()
 
+    @testing.fails_on('maxdb')
     def testorphan(self):
         jack = ctx.current.query(User).get_by(user_name='jack')
         p = jack.pref
@@ -242,6 +246,7 @@ class M2OCascadeTest(testbase.AssertMixin):
         assert p not in ctx.current
         assert e not in ctx.current
 
+    @testing.fails_on('maxdb')
     def testorphan2(self):
         jack = ctx.current.query(User).get_by(user_name='jack')
         p = jack.pref
@@ -257,28 +262,37 @@ class M2OCascadeTest(testbase.AssertMixin):
         ctx.current.flush()
         assert p not in ctx.current
         assert e not in ctx.current
-    
+
+    def testorphan3(self):
+        """test that double assignment doesn't accidentally reset the 'parent' flag."""
+        
+        jack = ctx.current.query(User).get_by(user_name='jack')
+        newpref = Pref("newpref")
+        jack.pref = newpref
+        jack.pref = newpref
+        ctx.current.flush()
         
 
-class M2MCascadeTest(testbase.AssertMixin):
+
+class M2MCascadeTest(AssertMixin):
     def setUpAll(self):
         global metadata, a, b, atob
         metadata = MetaData(testbase.db)
-        a = Table('a', metadata, 
+        a = Table('a', metadata,
             Column('id', Integer, primary_key=True),
             Column('data', String(30))
             )
-        b = Table('b', metadata,     
+        b = Table('b', metadata,
             Column('id', Integer, primary_key=True),
             Column('data', String(30))
             )
-        atob = Table('atob', metadata, 
+        atob = Table('atob', metadata,
             Column('aid', Integer, ForeignKey('a.id')),
             Column('bid', Integer, ForeignKey('b.id'))
-            
+
             )
         metadata.create_all()
-        
+
     def tearDownAll(self):
         metadata.drop_all()
 
@@ -289,26 +303,26 @@ class M2MCascadeTest(testbase.AssertMixin):
         class B(object):
             def __init__(self, data):
                 self.data = data
-        
+
         mapper(A, a, properties={
             # if no backref here, delete-orphan failed until [ticket:427] was fixed
             'bs':relation(B, secondary=atob, cascade="all, delete-orphan")
         })
         mapper(B, b)
-        
+
         sess = create_session()
         a1 = A('a1')
         b1 = B('b1')
         a1.bs.append(b1)
         sess.save(a1)
         sess.flush()
-        
+
         a1.bs.remove(b1)
         sess.flush()
         assert atob.count().scalar() ==0
         assert b.count().scalar() == 0
         assert a.count().scalar() == 1
-    
+
     def testcascadedelete(self):
         class A(object):
             def __init__(self, data):
@@ -316,28 +330,28 @@ class M2MCascadeTest(testbase.AssertMixin):
         class B(object):
             def __init__(self, data):
                 self.data = data
-        
+
         mapper(A, a, properties={
             'bs':relation(B, secondary=atob, cascade="all, delete-orphan")
         })
         mapper(B, b)
-        
+
         sess = create_session()
         a1 = A('a1')
         b1 = B('b1')
         a1.bs.append(b1)
         sess.save(a1)
         sess.flush()
-        
+
         sess.delete(a1)
         sess.flush()
         assert atob.count().scalar() ==0
         assert b.count().scalar() == 0
         assert a.count().scalar() == 0
 
-class UnsavedOrphansTest(testbase.ORMTest):
+class UnsavedOrphansTest(ORMTest):
     """tests regarding pending entities that are orphans"""
-    
+
     def define_tables(self, metadata):
         global users, addresses, User, Address
         users = Table('users', metadata,
@@ -352,7 +366,7 @@ class UnsavedOrphansTest(testbase.ORMTest):
         )
         class User(object):pass
         class Address(object):pass
-        
+
     def test_pending_orphan(self):
         """test that an entity that never had a parent on a delete-orphan cascade cant be saved."""
 
@@ -370,7 +384,7 @@ class UnsavedOrphansTest(testbase.ORMTest):
         assert a.address_id is None, "Error: address should not be persistent"
 
     def test_delete_new_object(self):
-        """test that an entity which is attached then detached from its 
+        """test that an entity which is attached then detached from its
         parent with a delete-orphan cascade gets counted as an orphan"""
 
         mapper(Address, addresses)
@@ -381,6 +395,7 @@ class UnsavedOrphansTest(testbase.ORMTest):
 
         u = User()
         s.save(u)
+        s.flush()
         a = Address()
         assert a not in s.new
         u.addresses.append(a)
@@ -391,11 +406,10 @@ class UnsavedOrphansTest(testbase.ORMTest):
             assert False
         except exceptions.FlushError:
             assert True
-        assert u.user_id is None, "Error: user should not be persistent"
         assert a.address_id is None, "Error: address should not be persistent"
 
 
-class UnsavedOrphansTest2(testbase.ORMTest):
+class UnsavedOrphansTest2(ORMTest):
     """same test as UnsavedOrphans only three levels deep"""
 
     def define_tables(self, meta):
@@ -419,7 +433,7 @@ class UnsavedOrphansTest2(testbase.ORMTest):
         )
 
     def testdeletechildwithchild(self):
-        """test that an entity which is attached then detached from its 
+        """test that an entity which is attached then detached from its
         parent with a delete-orphan cascade gets counted as an orphan, as well
         as its own child instances"""
 
@@ -455,9 +469,9 @@ class UnsavedOrphansTest2(testbase.ORMTest):
         assert item.id is None
         assert attr.id is None
 
-class DoubleParentOrphanTest(testbase.AssertMixin):
+class DoubleParentOrphanTest(AssertMixin):
     """test orphan detection for an entity with two parent relations"""
-    
+
     def setUpAll(self):
         global metadata, address_table, businesses, homes
         metadata = MetaData(testbase.db)
@@ -484,7 +498,7 @@ class DoubleParentOrphanTest(testbase.AssertMixin):
         metadata.drop_all()
     def test_non_orphan(self):
         """test that an entity can have two parent delete-orphan cascades, and persists normally."""
-        
+
         class Address(object):pass
         class Home(object):pass
         class Business(object):pass
@@ -501,7 +515,7 @@ class DoubleParentOrphanTest(testbase.AssertMixin):
         b1.address = a2
         [session.save(x) for x in [h1,b1]]
         session.flush()
-        
+
     def test_orphan(self):
         """test that an entity can have two parent delete-orphan cascades, and is detected as an orphan
         when saved without a parent."""
@@ -521,7 +535,61 @@ class DoubleParentOrphanTest(testbase.AssertMixin):
             assert False
         except exceptions.FlushError, e:
             assert True
-    
-            
+
+class CollectionAssignmentOrphanTest(AssertMixin):
+    def setUpAll(self):
+        global metadata, table_a, table_b
+
+        metadata = MetaData(testbase.db)
+        table_a = Table('a', metadata,
+                        Column('id', Integer, primary_key=True),
+                        Column('foo', String(30)))
+        table_b = Table('b', metadata,
+                        Column('id', Integer, primary_key=True),
+                        Column('foo', String(30)),
+                        Column('a_id', Integer, ForeignKey('a.id')))
+        metadata.create_all()
+
+    def tearDown(self):
+        clear_mappers()
+    def tearDownAll(self):
+        metadata.drop_all()
+
+    def test_basic(self):
+        class A(object):
+            def __init__(self, foo):
+                self.foo = foo
+        class B(object):
+            def __init__(self, foo):
+                self.foo = foo
+
+        mapper(A, table_a, properties={
+            'bs':relation(B, cascade="all, delete-orphan")
+            })
+        mapper(B, table_b)
+
+        a1 = A('a1')
+        a1.bs.append(B('b1'))
+        a1.bs.append(B('b2'))
+        a1.bs.append(B('b3'))
+
+        sess = create_session()
+        sess.save(a1)
+        sess.flush()
+
+        assert table_b.count(table_b.c.a_id == None).scalar() == 0
+
+        assert table_b.count().scalar() == 3
+
+        a1 = sess.query(A).get(a1.id)
+        assert len(a1.bs) == 3
+        a1.bs = list(a1.bs)
+        assert not class_mapper(B)._is_orphan(a1.bs[0])
+        a1.bs[0].foo='b2modified'
+        a1.bs[1].foo='b3modified'
+        sess.flush()
+
+        assert table_b.count().scalar() == 3
+
 if __name__ == "__main__":
-    testbase.main()        
+    testbase.main()
