@@ -1,11 +1,9 @@
-from testbase import PersistTest, AssertMixin
-import unittest
-from sqlalchemy import *
 import testbase
+from sqlalchemy import *
+from sqlalchemy.orm import *
 from sqlalchemy.ext.sessioncontext import SessionContext
-
-from tables import *
-import tables
+from testlib import *
+from testlib.tables import *
 
 class EntityTest(AssertMixin):
     """tests mappers that are constructed based on "entity names", which allows the same class
@@ -16,27 +14,36 @@ class EntityTest(AssertMixin):
         ctx = SessionContext(create_session)
         
         user1 = Table('user1', metadata, 
-            Column('user_id', Integer, Sequence('user1_id_seq'), primary_key=True),
+            Column('user_id', Integer, Sequence('user1_id_seq', optional=True),
+                   primary_key=True),
             Column('name', String(60), nullable=False)
             )
         user2 = Table('user2', metadata, 
-            Column('user_id', Integer, Sequence('user2_id_seq'), primary_key=True),
+            Column('user_id', Integer, Sequence('user2_id_seq', optional=True),
+                   primary_key=True),
             Column('name', String(60), nullable=False)
             )
         address1 = Table('address1', metadata,
-            Column('address_id', Integer, Sequence('address1_id_seq'), primary_key=True),
-            Column('user_id', Integer, ForeignKey(user1.c.user_id), nullable=False),
+            Column('address_id', Integer,
+                   Sequence('address1_id_seq', optional=True),
+                   primary_key=True),
+            Column('user_id', Integer, ForeignKey(user1.c.user_id),
+                   nullable=False),
             Column('email', String(100), nullable=False)
             )
         address2 = Table('address2', metadata,
-            Column('address_id', Integer, Sequence('address2_id_seq'), primary_key=True),
-            Column('user_id', Integer, ForeignKey(user2.c.user_id), nullable=False),
+            Column('address_id', Integer,
+                   Sequence('address2_id_seq', optional=True),
+                   primary_key=True),
+            Column('user_id', Integer, ForeignKey(user2.c.user_id),
+                   nullable=False),
             Column('email', String(100), nullable=False)
             )
         metadata.create_all()
     def tearDownAll(self):
         metadata.drop_all()
     def tearDown(self):
+        ctx.current.clear()
         clear_mappers()
         for t in metadata.table_iterator(reverse=True):
             t.delete().execute()
@@ -72,16 +79,21 @@ class EntityTest(AssertMixin):
         ctx.current.flush()
         assert user1.select().execute().fetchall() == [(u1.user_id, u1.name)]
         assert user2.select().execute().fetchall() == [(u2.user_id, u2.name)]
-        assert address1.select().execute().fetchall() == [(u1.user_id, a1.user_id, 'a1@foo.com')]
-        assert address2.select().execute().fetchall() == [(u2.user_id, a2.user_id, 'a2@foo.com')]
+        assert address1.select().execute().fetchall() == [(a1.address_id, u1.user_id, 'a1@foo.com')]
+        assert address2.select().execute().fetchall() == [(a1.address_id, u2.user_id, 'a2@foo.com')]
 
         ctx.current.clear()
-        u1list = ctx.current.query(User, entity_name='user1').select()
-        u2list = ctx.current.query(User, entity_name='user2').select()
+        u1list = ctx.current.query(User, entity_name='user1').all()
+        u2list = ctx.current.query(User, entity_name='user2').all()
         assert len(u1list) == len(u2list) == 1
         assert u1list[0] is not u2list[0]
         assert len(u1list[0].addresses) == len(u2list[0].addresses) == 1
 
+        u1 = ctx.current.query(User, entity_name='user1').first()
+        ctx.current.refresh(u1)
+        ctx.current.expire(u1)
+
+
     def testcascade(self):
         """same as testbasic but relies on session cascading"""
         class User(object):pass
@@ -111,12 +123,12 @@ class EntityTest(AssertMixin):
         u2.addresses.append(a2)
         sess.save(u2, entity_name='user2')
         print u2.__dict__
-        
-        sess.flush()
+
+        sess.flush()        
         assert user1.select().execute().fetchall() == [(u1.user_id, u1.name)]
         assert user2.select().execute().fetchall() == [(u2.user_id, u2.name)]
-        assert address1.select().execute().fetchall() == [(u1.user_id, a1.user_id, 'a1@foo.com')]
-        assert address2.select().execute().fetchall() == [(u2.user_id, a2.user_id, 'a2@foo.com')]
+        assert address1.select().execute().fetchall() == [(a1.address_id, u1.user_id, 'a1@foo.com')]
+        assert address2.select().execute().fetchall() == [(a1.address_id, u2.user_id, 'a2@foo.com')]
 
         sess.clear()
         u1list = sess.query(User, entity_name='user1').select()
@@ -155,8 +167,8 @@ class EntityTest(AssertMixin):
         ctx.current.flush()
         assert user1.select().execute().fetchall() == [(u1.user_id, u1.name)]
         assert user2.select().execute().fetchall() == [(u2.user_id, u2.name)]
-        assert address1.select().execute().fetchall() == [(u1.user_id, a1.user_id, 'a1@foo.com')]
-        assert address2.select().execute().fetchall() == [(u2.user_id, a2.user_id, 'a2@foo.com')]
+        assert address1.select().execute().fetchall() == [(a1.address_id, u1.user_id, 'a1@foo.com')]
+        assert address2.select().execute().fetchall() == [(a1.address_id, u2.user_id, 'a2@foo.com')]
 
         ctx.current.clear()
         u1list = ctx.current.query(User, entity_name='user1').select()
