@@ -7,14 +7,20 @@ def suite():
         'sql.testtypes',
         'sql.constraints',
 
+        'sql.generative',
+        
         # SQL syntax
         'sql.select',
         'sql.selectable',
         'sql.case_statement', 
+        'sql.labels',
+        'sql.unicode',
         
         # assorted round-trip tests
+        'sql.functions',
         'sql.query',
         'sql.quote',
+        'sql.rowcount',
         
         # defaults, sequences (postgres/oracle)
         'sql.defaults',
@@ -27,7 +33,5 @@ def suite():
         alltests.addTest(unittest.findTestCases(mod, suiteClass=None))
     return alltests
 
-
-
 if __name__ == '__main__':
-    testbase.runTests(suite())
+    testbase.main(suite())
