@@ -1,17 +1,13 @@
-"""tests that various From objects properly export their columns, as well as useable primary keys
-and foreign keys.  Full relational algebra depends on every selectable unit behaving
-nicely with others.."""
-
+"""tests that various From objects properly export their columns, as well as
+useable primary keys and foreign keys.  Full relational algebra depends on
+every selectable unit behaving nicely with others.."""
+ 
 import testbase
-import unittest, sys, datetime
-
-
-db = testbase.db
-
 from sqlalchemy import *
+from testlib import *
 
-
-table = Table('table1', db, 
+metadata = MetaData()
+table = Table('table1', metadata, 
     Column('col1', Integer, primary_key=True),
     Column('col2', String(20)),
     Column('col3', Integer),
@@ -19,15 +15,16 @@ table = Table('table1', db,
     
 )
 
-table2 = Table('table2', db,
+table2 = Table('table2', metadata,
     Column('col1', Integer, primary_key=True),
     Column('col2', Integer, ForeignKey('table1.col1')),
     Column('col3', String(20)),
     Column('coly', Integer),
 )
 
-class SelectableTest(testbase.AssertMixin):
+class SelectableTest(AssertMixin):
     def testdistance(self):
+        # same column three times
         s = select([table.c.col1.label('c2'), table.c.col1, table.c.col1.label('c1')])
 
         # didnt do this yet...col.label().make_proxy() has same "distance" as col.make_proxy() so far
@@ -51,13 +48,19 @@ class SelectableTest(testbase.AssertMixin):
         j2 = jjj.alias('foo')
         assert j2.corresponding_column(table.c.col1) is j2.c.table1_col1
         
-
+    def testselectontable(self):
+        sel = select([table, table2], use_labels=True)
+        assert sel.corresponding_column(table.c.col1) is sel.c.table1_col1
+        assert sel.corresponding_column(table.c.col1, require_embedded=True) is sel.c.table1_col1
+        assert table.corresponding_column(sel.c.table1_col1) is table.c.col1
+        assert table.corresponding_column(sel.c.table1_col1, require_embedded=True) is None
+        
     def testjoinagainstjoin(self):
         j  = outerjoin(table, table2, table.c.col1==table2.c.col2)
         jj = select([ table.c.col1.label('bar_col1')],from_obj=[j]).alias('foo')
         jjj = join(table, jj, table.c.col1==jj.c.bar_col1)
         assert jjj.corresponding_column(jjj.c.table1_col1) is jjj.c.table1_col1
-
+        
         j2 = jjj.alias('foo')
         print j2.corresponding_column(jjj.c.table1_col1)
         assert j2.corresponding_column(jjj.c.table1_col1) is j2.c.table1_col1
@@ -170,8 +173,46 @@ class SelectableTest(testbase.AssertMixin):
         print str(criterion)
         print str(j.onclause)
         self.assert_(criterion.compare(j.onclause))
+    
+    def testtablejoinedtoselectoftable(self):
+        metadata = MetaData()
+        a = Table('a', metadata,
+            Column('id', Integer, primary_key=True))
+        b = Table('b', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('aid', Integer, ForeignKey('a.id')),
+            )
 
-class PrimaryKeyTest(testbase.AssertMixin):
+        j1 = a.outerjoin(b)
+        j2 = select([a.c.id.label('aid')]).alias('bar')
+
+        j3 = a.join(j2, j2.c.aid==a.c.id)
+
+        j4 = select([j3]).alias('foo')
+        print j4
+        print j4.corresponding_column(j2.c.aid)
+        print j4.c.aid
+        assert j4.corresponding_column(j2.c.aid) is j4.c.aid
+        assert j4.corresponding_column(a.c.id) is j4.c.id
+
+    def test_oid(self):
+        # the oid column of a selectable currently proxies all
+        # oid columns found within.  
+        s = table.select()
+        s2 = table2.select()
+        s3 = select([s, s2])
+        assert s3.corresponding_column(table.oid_column) is s3.oid_column
+        assert s3.corresponding_column(table2.oid_column) is s3.oid_column
+        assert s3.corresponding_column(s.oid_column) is s3.oid_column
+        assert s3.corresponding_column(s2.oid_column) is s3.oid_column
+        
+        u = s.union(s2)
+        assert u.corresponding_column(table.oid_column) is u.oid_column
+        assert u.corresponding_column(table2.oid_column) is u.oid_column
+        assert u.corresponding_column(s.oid_column) is u.oid_column
+        assert u.corresponding_column(s2.oid_column) is u.oid_column
+        
+class PrimaryKeyTest(AssertMixin):
     def test_join_pk_collapse_implicit(self):
         """test that redundant columns in a join get 'collapsed' into a minimal primary key, 
         which is the root column along a chain of foreign key relationships."""
@@ -224,8 +265,48 @@ class PrimaryKeyTest(testbase.AssertMixin):
         j.foreign_keys
         assert list(j.primary_key) == [a.c.id]
 
+    def test_non_column_clause(self):
+        meta = MetaData()
+        a = Table('a', meta, Column('id', Integer, primary_key=True), Column('x', Integer))
+        b = Table('b', meta, Column('id', Integer, ForeignKey('a.id'), primary_key=True), Column('x', Integer, primary_key=True))
+
+        j = a.join(b, and_(a.c.id==b.c.id, b.c.x==5))
+        assert str(j) == "a JOIN b ON a.id = b.id AND b.x = :b_x_1", str(j)
+        assert list(j.primary_key) == [a.c.id, b.c.x]
+
+class DerivedTest(AssertMixin):
+    def test_table(self):
+        meta = MetaData()
+        t1 = Table('t1', meta, Column('c1', Integer, primary_key=True), Column('c2', String(30)))
+        t2 = Table('t2', meta, Column('c1', Integer, primary_key=True), Column('c2', String(30)))
         
+        assert t1.is_derived_from(t1)
+        assert not t2.is_derived_from(t1)
         
+    def test_alias(self):    
+        meta = MetaData()
+        t1 = Table('t1', meta, Column('c1', Integer, primary_key=True), Column('c2', String(30)))
+        t2 = Table('t2', meta, Column('c1', Integer, primary_key=True), Column('c2', String(30)))
+        
+        assert t1.alias().is_derived_from(t1)
+        assert not t2.alias().is_derived_from(t1)
+        assert not t1.is_derived_from(t1.alias())
+        assert not t1.is_derived_from(t2.alias())
+    
+    def test_select(self):
+        meta = MetaData()
+        t1 = Table('t1', meta, Column('c1', Integer, primary_key=True), Column('c2', String(30)))
+        t2 = Table('t2', meta, Column('c1', Integer, primary_key=True), Column('c2', String(30)))
+
+        assert t1.select().is_derived_from(t1)
+        assert not t2.select().is_derived_from(t1)
+
+        assert select([t1, t2]).is_derived_from(t1)
+
+        assert t1.select().alias('foo').is_derived_from(t1)
+        assert select([t1, t2]).alias('foo').is_derived_from(t1)
+        assert not t2.select().alias('foo').is_derived_from(t1)
+
 if __name__ == "__main__":
     testbase.main()
-    
\ No newline at end of file
+
